<?php
/**
 * @file
 * Contains functions only needed for drush integration.
 */

/**
 * Implementation of hook_drush_command().
 */
function zurb_template_drush_command() {
  $items = array();

  $items['zurb-template-sub-theme'] = array(
    'description' => 'Create a ZURB foundation sub-theme',
    'aliases' => array('zurb'),
    'arguments' => array(
      'name'         => 'Your sub-theme name.',
      'machine_name' => 'A machine-readable name for your theme, optional only  [a-z, 0-9] ',
    ),
    'options' => array(
      'description'  => 'Your sub-theme description.',
      'machine-name' => '[a-z, 0-9] A machine-readable name for your theme.'
    ),
    'examples' => array(
      'drush zurb "custom theme name"' => 'Create a sub-theme with the default options.',
      'drush zurb "foo bar" "foo_bar"  --description="My supersweet awesome theme"' => 'Create a sub-theme with additional options.',
    ),
  );

  return $items;
}

/**
 * Create a ZURB Template sub-theme.
 */
function drush_zurb_template_sub_theme($name = NULL, $machine_name = NULL, $description = NULL) {
  if (empty($name)) {
    drush_set_error(dt("Please provide a name for the sub-theme.\nUSAGE:\tdrush zurb [name] [machine_name !OPTIONAL] [description !OPTIONAL]\n"));
    return;
  }
  //Filter everything but letters, numbers, underscores, and hyphens
  $machine_name = !empty($machine_name) ? preg_replace('/[^a-z0-9_-]+/', '', strtolower($machine_name)) : preg_replace('/[^a-z0-9_-]+/', '', strtolower($name));
  // Eliminate hyphens
  $machine_name = str_replace('-', '_', $machine_name);

  $zurb_path = drush_get_context('DRUSH_DRUPAL_ROOT') . '/' . \Drupal::service('extension.list.theme')->getPath('zurb_template');
  $subtheme_path = explode('/', $zurb_path);
  array_pop($subtheme_path);
  $subtheme_path = implode('/', $subtheme_path) . '/' . $machine_name;

  // Make a fresh copy of the subtheme.
  $s = drush_copy_dir("$zurb_path/STARTER/", $subtheme_path);
  if (empty($s)) {
    return;
  }

  // Rename files and fill in the theme machine name
  drush_op('rename', "$subtheme_path/STARTER.breakpoints.yml", "$subtheme_path/$machine_name.breakpoints.yml");
  drush_op('rename', "$subtheme_path/STARTER.info.yml.txt", "$subtheme_path/$machine_name.info.yml");
  drush_op('rename', "$subtheme_path/STARTER.libraries.yml", "$subtheme_path/$machine_name.libraries.yml");
  drush_op('rename', "$subtheme_path/STARTER.theme", "$subtheme_path/$machine_name.theme");
  drush_op('rename', "$subtheme_path/config/install/STARTER.settings.yml.txt", "$subtheme_path/config/install/$machine_name.settings.yml");

  // Replaces instances of STARTER_MACHINE in required files to machine name of the theme.
  drush_op('zurb_template_file_str_replace', "$subtheme_path/package.json", 'STARTER_MACHINE', "$machine_name");

  // Replaces instances of STARTER in required files to name of the theme.
  drush_op('zurb_template_file_str_replace', "$subtheme_path/config.yml", 'STARTER', "$name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/gulpfile.babel.js", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/package.json", 'STARTER', "$name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/$machine_name.breakpoints.yml", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/$machine_name.info.yml", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/$machine_name.libraries.yml", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/$machine_name.theme", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/config/install/$machine_name.settings.yml", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/theme-settings.php", 'STARTER', "$machine_name");
  drush_op('zurb_template_file_str_replace', "$subtheme_path/inc/form.inc", 'STARTER', "$machine_name");

  // Change the name of the theme.
  drush_op('zurb_template_file_str_replace', "$subtheme_path/$machine_name.info.yml", 'ZURB Template Sub-Theme', "$name");

  // Change the name of the theme.
  if (!empty($description)) {
    drush_op('zurb_template_file_str_replace', "$subtheme_path/$machine_name.info.yml", 'A Drupal 8 theme based on ZURB Template powered by ZURB Foundation 6. Inherits from the ZURB Template base theme.', $description);
  }

  // Notify user of the newly created theme.
  drush_print(dt("\nSuccess! Your sub-theme !name was created at !path. To enable your new theme visit /admin/appearance.\n",
    array(
      '!name' => $name,
      '!path' => $subtheme_path,
    )
  ));
}

/**
 * Internal helper: Replace strings in a file.
 */
function zurb_template_file_str_replace($file_path, $find, $replace) {
  $file_contents = file_get_contents($file_path);
  $file_contents = str_replace($find, $replace, $file_contents);
  file_put_contents($file_path, $file_contents);
}

/**
 * Implements hook_drush_help().
 */
function zurb_template_drush_help($section) {
  switch ($section) {
    case 'drush:zurb-template-sub-theme':
      return dt("Create a ZURB Template sub-theme.");
  }
}
